﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
//
using System.IO;
using NAudio.Wave;

namespace Call.Tools
{
    //کلاس میانه برای استفاده و تبدیل فایلهای صوتی 
    public static class NAudioUtility
    {
        //تبدیل یک فایل به فایل wav
        public static int SaveToWaveFile(string InputFile, string OutputFile, int SampleRate = 8000, int Bit = 16, int Channels = 1, int Quality = 60)
        {
            List<string> InputFiles = new List<string>();
            InputFiles.Add(InputFile);

            return SaveToWaveFile(InputFiles, OutputFile, SampleRate, Bit, Channels, Quality);
        }

        //تبدیل چند فایل به فایل wav
        public static int SaveToWaveFile(List<string> InputFiles, string OutputFile, int SampleRate = 8000, int Bit = 16, int Channels = 1, int Quality = 60)
        {
            int success = 0;

            try
            {
                //لیست برای ذخیره سازی فایلهای تبدیل شده در حافظه موقت
                List<MemoryStream> lstEncodedStreams = new List<MemoryStream>();

                //پیمایش در لیست فایلهای ورودی
                foreach (string InputFile in InputFiles)
                {
                    if (File.Exists(InputFile))
                    {
                        bool ReaderIsFind = false;
                        WaveStream reader = null;

                        //خواندن فایلهای صوتی بر اساس فرمت خاص خود و تبدیل به فرمت استاندارد wav
                        //=>
                        if (new FileInfo(InputFile).Extension.ToLower().Trim() == ".wav")
                        {
                            try
                            {
                                reader = new WaveFileReader(InputFile);
                                ReaderIsFind = true;
                            }
                            catch
                            {
                            }
                        }

                        if (new FileInfo(InputFile).Extension.ToLower().Trim() == ".mp3")
                        {
                            try
                            {
                                reader = new Mp3FileReader(InputFile);
                                ReaderIsFind = true;
                            }
                            catch
                            {
                            }
                        }

                        if (new FileInfo(InputFile).Extension.ToLower().Trim() == ".wma")
                        {
                            try
                            {
                                reader = new NAudio.WindowsMediaFormat.WMAFileReader(InputFile);
                                ReaderIsFind = true;
                            }
                            catch
                            {
                            }
                        }

                        //فرمت flac
                        //نیاز به فریمورک 4/5 دارد
                        //در باقی موارد نیازی به فریمورک 4/5 نیست و میشود حداقل از فریمورک 4 استفاده کرد
                        if (new FileInfo(InputFile).Extension.ToLower().Trim() == ".flac")
                        {
                            try
                            {
                                reader = new NAudio.Flac.FlacReader(InputFile);
                                ReaderIsFind = true;
                            }
                            catch
                            {
                            }
                        }

                        if (new FileInfo(InputFile).Extension.ToLower().Trim() == ".ogg" ||
                            new FileInfo(InputFile).Extension.ToLower().Trim() == ".oga" ||
                            new FileInfo(InputFile).Extension.ToLower().Trim() == ".opus")
                        {
                            try
                            {
                                reader = new NAudio.Vorbis.VorbisWaveReader(InputFile);
                                ReaderIsFind = true;
                            }
                            catch
                            {
                            }
                        }

                        if (new FileInfo(InputFile).Extension.ToLower().Trim() == ".aiff" ||
                            new FileInfo(InputFile).Extension.ToLower().Trim() == ".aif" ||
                            new FileInfo(InputFile).Extension.ToLower().Trim() == ".aifc")
                        {
                            try
                            {
                                reader = new AiffFileReader(InputFile);
                                ReaderIsFind = true;
                            }
                            catch
                            {
                            }
                        }

                        //دیگر فرمت ها اگر قابل پشتیبانی باشد
                        //به تشخیص این دو متد کلی
                        if (ReaderIsFind == false)
                        {
                            try
                            {
                                reader = new MediaFoundationReader(InputFile);
                                ReaderIsFind = true;
                            }
                            catch
                            {
                                try
                                {
                                    reader = new AudioFileReader(InputFile);
                                    ReaderIsFind = true;
                                }
                                catch
                                {
                                }
                            }
                        }

                        //<=
                        //خواندن فایلهای صوتی بر اساس فرمت خاص خود و تبدیل به فرمت استاندارد wav

                        //اگر موفق به خواندن فایل شد
                        if (ReaderIsFind && reader != null)
                        {
                            //تنظیم اولیه فایل برای تبدیل
                            WaveFormat outFormat = new WaveFormat(SampleRate, Bit, Channels);
                            using (MediaFoundationResampler resampler = new MediaFoundationResampler(reader, outFormat))
                            {
                                //بازه کیفیت بین 1 و 60 میباشد
                                if (Quality > 60) Quality = 60;
                                if (Quality < 1) Quality = 1;

                                //تنظیم کیفیت
                                resampler.ResamplerQuality = Quality;

                                //فایل (استریم) حافظه موقت جدید برای ذخیره فایل صوتی تبدیل شده
                                MemoryStream ms = new MemoryStream();
                                WaveFileWriter.WriteWavFileToStream(ms, resampler);

                                //افزودن فایل تبدیل شده به لیست حافظه موقت
                                lstEncodedStreams.Add(ms);
                            }
                        }
                    }
                }

                //ترکیب فایلهای تبدیل شده به یک فایل و ذخیره در سیستم فایل
                byte[] buffer = new byte[1024];
                WaveFileWriter waveFileWriter = null;
                try
                {
                    //پیمایش در تمام فایلهای تبدیل شده در حافظه موقت
                    foreach (MemoryStream ms in lstEncodedStreams)
                    {
                        using (StreamMediaFoundationReader reader = new StreamMediaFoundationReader(ms))
                        {
                            if (waveFileWriter == null)
                            {
                                //اگر فایل خروجی از قبل موجود باشد ابتدا آن را پاک کند
                                if (File.Exists(OutputFile))
                                {
                                    File.Delete(OutputFile);
                                }
                                waveFileWriter = new WaveFileWriter(OutputFile, reader.WaveFormat);
                            }
                            else
                            {
                                //احتمال صفر در این حالت چون همه فایلهای استریم یک فرمت دارند
                                //برای محکم کاری ;)
                                if (!reader.WaveFormat.Equals(waveFileWriter.WaveFormat))
                                {
                                    throw new InvalidOperationException("Can't concatenate WAV Files that don't share the same format");
                                }
                            }

                            //افزودن فایلها به یک فایل
                            int read;
                            while ((read = reader.Read(buffer, 0, buffer.Length)) > 0)
                            {
                                waveFileWriter.Write(buffer, 0, read);
                            }

                            //افزودن به تعداد فایلهایی که موفق به ترکیب شدند 
                            success++;
                        }
                    }
                }
                finally
                {
                    if (waveFileWriter != null)
                    {
                        waveFileWriter.Dispose();
                    }
                }
            }
            catch
            {
            }

            return success;
        }
    }
}