﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Linq;

namespace Call
{
    public partial class frmMergeAudioFiles : Form
    {
        List<Models.AudioFile> FileNames;
        public frmMergeAudioFiles()
        {
            InitializeComponent();

            FileNames = new List<Models.AudioFile>();
            DGV.DataSource = FileNames.ToList();

            FillComboBox();
        }

        private void FillComboBox()
        {
            var lstBit = new List<Models.ListItem>();

            lstBit.Add(new Models.ListItem() { Text = "16", Value = 16 });
            lstBit.Add(new Models.ListItem() { Text = "32", Value = 32 });

            ddlBit.DisplayMember = "Text";
            ddlBit.ValueMember = "Value";
            ddlBit.DataSource = lstBit;

            var lstChannels = new List<Models.ListItem>();

            lstChannels.Add(new Models.ListItem() { Text = "1", Value = 1 });
            lstChannels.Add(new Models.ListItem() { Text = "2", Value = 2 });

            ddlChannels.DisplayMember = "Text";
            ddlChannels.ValueMember = "Value";
            ddlChannels.DataSource = lstChannels;


            var lstSampleRate = new List<Models.ListItem>();

            lstSampleRate.Add(new Models.ListItem() { Text = "8khz", Value = 8000 });
            lstSampleRate.Add(new Models.ListItem() { Text = "11khz", Value = 11000 });
            lstSampleRate.Add(new Models.ListItem() { Text = "12khz", Value = 12000 });
            lstSampleRate.Add(new Models.ListItem() { Text = "16khz", Value = 16000 });
            lstSampleRate.Add(new Models.ListItem() { Text = "22khz", Value = 22000 });
            lstSampleRate.Add(new Models.ListItem() { Text = "24khz", Value = 24000 });
            lstSampleRate.Add(new Models.ListItem() { Text = "32khz", Value = 32000 });
            lstSampleRate.Add(new Models.ListItem() { Text = "44.1khz", Value = 44100 });
            lstSampleRate.Add(new Models.ListItem() { Text = "48khz", Value = 48000 });
            lstSampleRate.Add(new Models.ListItem() { Text = "96khz", Value = 96000 });
            lstSampleRate.Add(new Models.ListItem() { Text = "192khz", Value = 192000 });

            ddlSampleRate.DisplayMember = "Text";
            ddlSampleRate.ValueMember = "Value";
            ddlSampleRate.DataSource = lstSampleRate;
        }

        private void btnAddFile_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                this.FileNames.AddRange(openFileDialog1.FileNames.ToList().Select(r => new Models.AudioFile()
                {
                    FilePath = r,
                    FileName = new System.IO.FileInfo(r).Name,
                    FileExtention = new System.IO.FileInfo(r).Extension,
                    FileSize = ((Convert.ToDouble(new System.IO.FileInfo(r).Length) / 1024) / 1024).ToString("N2"),
                }));
                this.FileNames = this.FileNames.GroupBy(r => r.FilePath).Select(r => r.First()).ToList();
                DGV.DataSource = FileNames.ToList();
            }
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            if (DGV.SelectedRows.Count == 1)
            {
                var dgvRow = DGV.SelectedRows[0];
                if (dgvRow != null)
                {
                    int idx = DGV.SelectedRows[0].Index;
                    this.FileNames.RemoveAt(idx);

                    this.FileNames = this.FileNames.GroupBy(r => r.FilePath).Select(r => r.First()).ToList();
                    DGV.DataSource = FileNames.ToList();

                    idx = idx - 1;
                    if (idx < 0) idx = 0;
                    if (this.FileNames.Count > 0)
                        DGV.Rows[idx].Selected = true;
                }
            }
        }

        private void btnDown_Click(object sender, EventArgs e)
        {
            if (DGV.SelectedRows.Count == 1)
            {
                var dgvRow = DGV.SelectedRows[0];
                if (dgvRow != null)
                {
                    int idx = DGV.SelectedRows[0].Index;

                    if ((idx != -1) && (idx < DGV.Rows.Count - 1))
                    {
                        this.FileNames = (List<Models.AudioFile>)DGV.DataSource;

                        var value = this.FileNames[idx];
                        this.FileNames.Remove(value);
                        this.FileNames.Insert(idx + 1, value);

                        this.FileNames = this.FileNames.GroupBy(r => r.FilePath).Select(r => r.First()).ToList();
                        DGV.DataSource = FileNames.ToList();

                        DGV.Rows[idx + 1].Selected = true;
                    }
                }
            }
        }

        private void btnUp_Click(object sender, EventArgs e)
        {
            if (DGV.SelectedRows.Count == 1)
            {
                var dgvRow = DGV.SelectedRows[0];
                if (dgvRow != null)
                {
                    int idx = DGV.SelectedRows[0].Index;

                    if (idx > 0)
                    {
                        this.FileNames = (List<Models.AudioFile>)DGV.DataSource;

                        var value = this.FileNames[idx];

                        this.FileNames.Insert(idx - 1, value);

                        this.FileNames.RemoveAt(idx + 1);

                        this.FileNames = this.FileNames.GroupBy(r => r.FilePath).Select(r => r.First()).ToList();
                        DGV.DataSource = FileNames.ToList();

                        DGV.Rows[idx - 1].Selected = true;
                    }
                }
            }
        }

        private void btnMergeAndSave_Click(object sender, EventArgs e)
        {
            if (this.FileNames.Count > 0)
            {
                saveFileDialog1.FileName = "Wave_" + DateTime.Now.ToString("yyyyMMddHHmmss") + ".wav";

                if (saveFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    this.Text = "لطفا منتظر بمانید";
                    this.Enabled = false;

                    int MergeCount;
                    var InputFiles = this.FileNames.Select(r => r.FilePath).ToList();
                    var SavedFilePath = saveFileDialog1.FileName;

                    if (radioDefault.Checked)
                    {
                        MergeCount = Call.Tools.NAudioUtility.SaveToWaveFile(InputFiles, SavedFilePath);
                    }
                    else
                    {
                        var SampleRate = Convert.ToInt32(ddlSampleRate.SelectedValue);
                        var Bit = Convert.ToInt32(ddlBit.SelectedValue);
                        var Channels = Convert.ToInt32(ddlChannels.SelectedValue);
                        var Quality = Convert.ToInt32(numUDQuality.Value);

                        MergeCount = Call.Tools.NAudioUtility.SaveToWaveFile(InputFiles, SavedFilePath, SampleRate, Bit, Channels, Quality);
                    }

                    this.Text = "ترکیب و تبدیل فایلهای صوتی";
                    this.Enabled = true;

                    if (MergeCount == 0)
                    {
                        if (System.IO.File.Exists(SavedFilePath))
                        {
                            System.IO.File.Delete(SavedFilePath);
                        }
                        MessageBox.Show("فایلی ذخیره نشد", "پیام", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    else if (InputFiles.Count == 1)
                    {
                        MessageBox.Show("فایل با موفقیت ذخیره شد", "پیام", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                    else
                    {
                        MessageBox.Show("تعداد " + MergeCount + " فایل با موفقیت ترکیب و ذخیره شد", "پیام", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                }
            }
            else
            {
                MessageBox.Show("فایلی انتخاب نشده است", "پیام", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void radioShared_Click(object sender, EventArgs e)
        {
            groupBoxCustom.Enabled = radioCustom.Checked;
            if (radioDefault.Checked)
            {
                ddlBit.SelectedIndex = ddlChannels.SelectedIndex = ddlSampleRate.SelectedIndex = 0;
                numUDQuality.Value = 60;
            }
        }

        private void DGV_DataBindingComplete(object sender, DataGridViewBindingCompleteEventArgs e)
        {
            lblCount.Text = this.FileNames.Count.ToString("N0");
        }
    }
}
